/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.Queue
 *
 * de.bsvrz.kex.tls.osi7.Queue is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.Queue is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.Queue.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.Queue.
 *
 *  de.bsvrz.kex.tls.osi7.Queue ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.Queue wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7;

import java.util.*;

/**
 * Implementierung einer LIFO-Queue fr {@link DeBlockToSend}
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kni (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public final class Queue {
	// ------------------------------------------------------------------------
	// VARIABLENDEKLARATIONEN
	// ------------------------------------------------------------------------

	// Private ----------------------------------------------------------------
	/** Die Queue */
	private final LinkedList<DeBlockToSend> _queue = new LinkedList<DeBlockToSend>();

	// ------------------------------------------------------------------------
	// GETTER / SETTER METHODEN
	// ------------------------------------------------------------------------

	/**
	 * Testet, ob noch Elemente in der Queue enthalten sind.
	 *
	 * @return true, wenn noch Elemente in der Queue enthalten sind, sonst false.
	 */
	public boolean isEmpty() {
		return _queue.isEmpty();
	}

	// ------------------------------------------------------------------------
	// SONSTIGE METHODEN
	// ------------------------------------------------------------------------

	/**
	 * Element aus der LIFO-Liste entnehmen
	 *
	 * @return {@link DeBlockToSend}, welches aus der LIFO-Liste entfernt wurde
	 */
	public DeBlockToSend pop() {
		synchronized (_queue) {
			try {
				while (_queue.isEmpty()) _queue.wait();
			}
			catch (InterruptedException e) {
				return null;
			}
			return _queue.removeLast();
		}
	}

	/**
	 * Alle Element aus der LIFO-Liste entnehmen (ohne warten)
	 *
	 * @return {@link DeBlockToSend}, welches aus der LIFO-Liste entfernt wurde
	 */
	//	public DeBlockToSend[] popAll() {
	public List<DeBlockToSend> popAll() {
		List<DeBlockToSend> list = new ArrayList<DeBlockToSend>();
		synchronized (_queue) {
			try {
				while (_queue.isEmpty()) _queue.wait();
			}
			catch (InterruptedException e) {
				return null;
			}
			// Etwas warten, damit eventuell noch mehrere De-Blcke in die
			// Queue kommen knnen
		}
		try {
			Thread.sleep(1000);
		}
		catch (InterruptedException e) {
			// nichts tun
		}

		synchronized (_queue) {
			while (!_queue.isEmpty()) {
				list.add(_queue.removeLast());
			}
		}
		//		DeBlockToSend[] t = new DeBlockToSend[list.size()];
		//		return (DeBlockToSend[]) list.toArray(t);
		return list;
	}

	/**
	 * Element in die LIFO-Liste schreiben
	 *
	 * @param deBlockToSend {@link DeBlockToSend}, welches in die LIFO-Liste geschrieben wird.
	 */
	public void push(DeBlockToSend deBlockToSend) {
		synchronized (_queue) {
			_queue.addFirst(deBlockToSend);
			_queue.notifyAll();
		}
	}

	/**
	 * Elemente in die LIFO-Liste schreiben
	 *
	 * @param deBlockToSend {@link DeBlockToSend}, welches in die LIFO-Liste geschrieben wird.
	 */
	public void push(DeBlockToSend[] deBlockToSend) {
		synchronized (_queue) {
			for (DeBlockToSend d : deBlockToSend) {
				_queue.addFirst(d);
			}
			_queue.notifyAll();
		}
	}
}


