/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.DeBlocksMultiBlockTransferDefault
 *
 * de.bsvrz.kex.tls.osi7.conversion.DeBlocksMultiBlockTransferDefault is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.DeBlocksMultiBlockTransferDefault is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.DeBlocksMultiBlockTransferDefault.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.DeBlocksMultiBlockTransferDefault.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.DeBlocksMultiBlockTransferDefault ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.DeBlocksMultiBlockTransferDefault wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */



package de.bsvrz.kex.tls.osi7.conversion;

//~ KLASSEN ===================================================================

/**
 * Legt die Liste der FG/Typ/ID Kombinationen fest, f&uuml;r die die Daten mit mehreren DE-Bl&ouml;cken
 * und der gleichen Jobnummer versendet werden m&uuml;ssen; z. B. TLS2009 Dateitransfers (FG254, Typ 41).<br>
 * Die Definition einer FG/Typ/ID Kombination in dieser Klasse f&uuml;hrt dazu, dass die einzelnen DE-Bl&ouml;cke
 * zun&auml;chst gesammelt werden, bis sie vollstndig sind. Dann werden alle diese Bl&ouml;cke mit der gleichen
 * Jobnummer versendet.
 * Die Struktur der DE-Bl&ouml;cke muss bez&uuml;glich der Blocknummern-Informationen wie folgt aufgebaut sein:<br>
 * (entsprechend TLS2009 Dateitransfers, FG254, Typ 41:<br>
 *      <pre>
 * Erster Block:
 *
 * Position         Bezeichnung                             Erl&auml;uterung
 * Byte 1           L&auml;nge DE-Block                     L&auml;nge des folgenden DE-Blocks
 * Byte 2           Daten-Endger&auml;te-Kanal              [ 0..255 ]
 * Byte 3           Typ der DE-Daten
 * Byte 4           Blocknummer low Byte                    [ 1 ]
 * Byte 5           Blocknummer high Byte
 * Byte 6           Anzahl Datenbl&ouml;cke low Byte        [ 2..65535 ]
 * Byte 7           Anzahl Datenbl&ouml;cke high Byte
 * Byte 8                                                   Daten
 *
 * Weitere Bl&ouml;cke:
 *
 * Position         Bezeichnung                             Erl&auml;uterung
 * Byte 1           L&auml;nge DE-Block                     L&auml;nge des folgenden DE-Blocks
 * Byte 2           Daten-Endger&auml;te-Kanal              [ 0..255 ]
 * Byte 3           Typ der DE-Daten
 * Byte 4           Blocknummer low Byte                    [ 0,2..65535 ]
 * Byte 5           Blocknummer high Byte
 * Byte 6                                                   Daten
 * </pre>
 *
 * Weiterhin gelten die Festlegungen TLS2009 Dateitransfers, FG254, Typ 41 zum Abbruch des Transfers (Blocknummer == 0).
 * @author BitCtrl Systems GmbH, Gieseler
 * @author Dipl.-Ing. Hans Christian Kni (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class DeBlocksMultiBlockTransferDefault {

    /**
     * Enthlt je Eintrag jeweils Funktionsgruppe, Typ und ID,
     * bei dem eine Datei (unendlich groer DE-Block) in mehreren
     * DE-Blcken (gem TLS) mit einer EINHEITLICHEN Jobnummer versendet
     * werden.
     */
    private static final int[][] deBlocksMultiBlockTransfer = new int[][] {

        // FG, Typ, ID
        { 254, 41, 3 },  // Dateitransfer (TLS 2009)
    };

    //~ GET METHODEN ==========================================================

    /**
     * Gibt die definierten FG/Typ/ID Kombinationen zur&uuml;ck.
     *
     * @return  Array mit den Eintrgen der MultiBlockDE-Typen
     */
    public static int[][] getDeBlocksMultiBlockTransfer() {
        return deBlocksMultiBlockTransfer;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 11.09.09
