/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.Fg001VerkehrsDatenTest
 *
 * de.bsvrz.kex.tls.osi7.conversion.Fg001VerkehrsDatenTest is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg001VerkehrsDatenTest is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.Fg001VerkehrsDatenTest.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.Fg001VerkehrsDatenTest.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg001VerkehrsDatenTest ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg001VerkehrsDatenTest wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */



package de.bsvrz.kex.tls.osi7.conversion;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.kex.tls.osi7.DeBlock;
import de.bsvrz.kex.tls.osi7.MessageSender;
import de.bsvrz.kex.tls.osi7.TlsData;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageState;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import java.util.HashSet;
import java.util.Set;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * ToDo: Kurzbeschreibung.
 * <p/>
 * ToDo: Langbeschreibung.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniss (HCK)
 */
public class Fg001VerkehrsDatenTest {

    //~ SET METHODEN ==========================================================

    @Before
    public void setUp() throws Exception {}

    //~ METHODEN ==============================================================

    @After
    public void tearDown() throws Exception {}

    @Test
    public void test() throws Exception {}

    @Test
    public void test_das_Konstruktor_mit_Typ113_16Bit_korrekt_initialisiert() throws Exception {
        int typ  = 113;   // Typ 113 (Option 0, 16 Bit)
        int qKfz = 1024;  // qKfz
        int qLkw = 999;   // qLkw
        int vPkw = 155;   // vPkw
        int vLkw = 82;    // vLkw

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw                                        // vLkw
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ113_16Bit_und_Fehlerwerten_korrekt_initialisiert() throws Exception {
        int typ  = 113;     // Typ 113 (Option 0, 16 Bit)
        int qKfz = 0xffff;  // qKfz
        int qLkw = 0xffff;  // qLkw
        int vPkw = 255;     // vPkw
        int vLkw = 255;     // vLkw

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw                                        // vLkw
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ114_16Bit_korrekt_initialisiert() throws Exception {
        int typ    = 114;   // Typ 114 (Option 1, 16 Bit)
        int qKfz   = 1024;  // qKfz
        int qLkw   = 999;   // qLkw
        int vPkw   = 155;   // vPkw
        int vLkw   = 82;    // vLkw
        int tNetto = 128;   // tNetto

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto                                      // tNetto
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(tNetto * 100));  // Angabe ist in ms
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ114_16Bit_und_Fehlerwerten_korrekt_initialisiert() throws Exception {
        int typ    = 114;     // Typ 114 (Option 1, 16 Bit)
        int qKfz   = 0xffff;  // qKfz
        int qLkw   = 0xffff;  // qLkw
        int vPkw   = 255;     // vPkw
        int vLkw   = 255;     // vLkw
        int tNetto = 255;     // tNetto

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto                                      // tNetto
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ115_16Bit_korrekt_initialisiert() throws Exception {
        int typ  = 115;   // Typ 115 (Option 2, 16 Bit)
        int qKfz = 1024;  // qKfz
        int qLkw = 999;   // qLkw
        int vPkw = 155;   // vPkw
        int vLkw = 82;    // vLkw
        int b    = 77;    // b

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) b                                           // b
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(b));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ115_16Bit_mit_fehlerhafter_Belegung_korrekt_initialisiert() throws Exception {
        int typ  = 115;   // Typ 115 (Option 2, 16 Bit)
        int qKfz = 1024;  // qKfz
        int qLkw = 999;   // qLkw
        int vPkw = 155;   // vPkw
        int vLkw = 82;    // vLkw
        int b    = 128;   // b > 100%

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) b                                           // b
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-3));  // b > 100%, deshalb auf nicht ermittelbar/fehlerhaft
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ115_16Bit_und_Fehlerwerten_korrekt_initialisiert() throws Exception {
        int typ  = 115;     // Typ 115 (Option 2, 116 Bit)
        int qKfz = 0xffff;  // qKfz
        int qLkw = 0xffff;  // qLkw
        int vPkw = 255;     // vPkw
        int vLkw = 255;     // vLkw
        int b    = 255;     // b

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) b                                           // b
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ116_16Bit_korrekt_initialisiert() throws Exception {
        int typ    = 116;   // Typ 116 (Option 3, 16 Bit)
        int qKfz   = 1024;  // qKfz
        int qLkw   = 999;   // qLkw
        int vPkw   = 155;   // vPkw
        int vLkw   = 82;    // vLkw
        int tNetto = 128;   // tNetto
        int b      = 77;    // b
        int sKfz   = 21;    // sKfz
        int vgKfz  = 134;   // vgKfz

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto,                                     // tNetto
            (byte) b,                                          // b
            (byte) sKfz,                                       // sKfz
            (byte) vgKfz                                       // vgKfz
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(tNetto * 100));  // Angabe ist in ms
        assertThat(fg001VerkehrsDaten.getB(), is(b));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(sKfz));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(vgKfz));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ116_16Bit_mit_fehlerhafter_Belegung_korrekt_initialisiert() throws Exception {
        int typ    = 116;   // Typ 116 (Option 3, 16 Bit)
        int qKfz   = 1024;  // qKfz
        int qLkw   = 999;   // qLkw
        int vPkw   = 155;   // vPkw
        int vLkw   = 82;    // vLkw
        int tNetto = 128;   // tNetto
        int b      = 128;   // b > 100%
        int sKfz   = 21;    // sKfz
        int vgKfz  = 134;   // vgKfz

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto,                                     // tNetto
            (byte) b,                                          // b
            (byte) sKfz,                                       // sKfz
            (byte) vgKfz                                       // vgKfz
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(tNetto * 100));  // Angabe ist in ms
        assertThat(fg001VerkehrsDaten.getB(), is(-3));                 // b > 100%, deshalb auf nicht ermittelbar/fehlerhaft
        assertThat(fg001VerkehrsDaten.getsKfz(), is(sKfz));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(vgKfz));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ116_16Bit_und_Fehlerwerten_korrekt_initialisiert() throws Exception {
        int typ    = 117;     // Typ 117 (Option 3, 16 Bit)
        int qKfz   = 0xffff;  // qKfz
        int qLkw   = 0xffff;  // qLkw
        int vPkw   = 255;     // vPkw
        int vLkw   = 255;     // vLkw
        int tNetto = 255;     // tNetto
        int b      = 255;     // b
        int sKfz   = 255;     // sKfz
        int vgKfz  = 255;     // vgKfz

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto,                                     // tNetto
            (byte) b,                                          // b
            (byte) sKfz,                                       // sKfz
            (byte) vgKfz                                       // vgKfz
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ117_16Bit_korrekt_initialisiert() throws Exception {
        int typ    = 117;   // Typ 117 (Option 4, 16 Bit)
        int qKfz   = 1024;  // qKfz
        int qLkw   = 999;   // qLkw
        int vPkw   = 155;   // vPkw
        int vLkw   = 82;    // vLkw
        int tNetto = 128;   // tNetto
        int b      = 77;    // b
        int sKfz   = 21;    // sKfz
        int vgKfz  = 134;   // vgKfz

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto,                                     // tNetto
            (byte) b,                                          // b
            (byte) sKfz,                                       // sKfz
            (byte) vgKfz                                       // vgKfz
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(tNetto * 100));  // Angabe ist in ms
        assertThat(fg001VerkehrsDaten.getB(), is(b));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(sKfz));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(vgKfz));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ117_16Bit_mit_fehlerhafter_Belegung_korrekt_initialisiert() throws Exception {
        int typ    = 117;   // Typ 117 (Option 4, 16 Bit)
        int qKfz   = 1024;  // qKfz
        int qLkw   = 999;   // qLkw
        int vPkw   = 155;   // vPkw
        int vLkw   = 82;    // vLkw
        int tNetto = 128;   // tNetto
        int b      = 128;   // b > 100%
        int sKfz   = 21;    // sKfz
        int vgKfz  = 134;   // vgKfz

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto,                                     // tNetto
            (byte) b,                                          // b
            (byte) sKfz,                                       // sKfz
            (byte) vgKfz                                       // vgKfz
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(tNetto * 100));  // Angabe ist in ms
        assertThat(fg001VerkehrsDaten.getB(), is(-3));                 // b > 100%, deshalb auf nicht ermittelbar/fehlerhaft
        assertThat(fg001VerkehrsDaten.getsKfz(), is(sKfz));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(vgKfz));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ117_16Bit_und_Fehlerwerten_korrekt_initialisiert() throws Exception {
        int typ    = 117;     // Typ 117 (Option 4, 16 Bit)
        int qKfz   = 0xffff;  // qKfz
        int qLkw   = 0xffff;  // qLkw
        int vPkw   = 255;     // vPkw
        int vLkw   = 255;     // vLkw
        int tNetto = 255;     // tNetto
        int b      = 255;     // b
        int sKfz   = 255;     // sKfz
        int vgKfz  = 255;     // vgKfz

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) (qKfz & 0x00ff),                            // qKfz
            (byte) ((qKfz & 0xff00) >> 8),                     // qKfz
            (byte) (qLkw & 0x00ff),                            // qLkw
            (byte) ((qLkw & 0xff00) >> 8),                     // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto,                                     // tNetto
            (byte) b,                                          // b
            (byte) sKfz,                                       // sKfz
            (byte) vgKfz                                       // vgKfz
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ49_8Bit_korrekt_initialisiert() throws Exception {
        int typ  = 49;   // Typ 49 (Option 0, 8 Bit)
        int qKfz = 20;   // qKfz
        int qLkw = 9;    // qLkw
        int vPkw = 155;  // vPkw
        int vLkw = 82;   // vLkw

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw                                        // vLkw
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ49_8Bit_und_Fehlerwerten_korrekt_initialisiert() throws Exception {
        int typ  = 49;   // Typ 49 (Option 0, 8 Bit)
        int qKfz = 255;  // qKfz
        int qLkw = 255;  // qLkw
        int vPkw = 255;  // vPkw
        int vLkw = 255;  // vLkw

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw                                        // vLkw
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ50_8Bit_korrekt_initialisiert() throws Exception {
        int typ    = 50;   // Typ 50 (Option 1, 8 Bit)
        int qKfz   = 20;   // qKfz
        int qLkw   = 9;    // qLkw
        int vPkw   = 155;  // vPkw
        int vLkw   = 82;   // vLkw
        int tNetto = 128;  // tNetto

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto                                      // tNetto
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(tNetto * 100));  // Angabe ist in ms
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ50_8Bit_und_Fehlerwerten_korrekt_initialisiert() throws Exception {
        int typ    = 50;   // Typ 50 (Option 1, 8 Bit)
        int qKfz   = 255;  // qKfz
        int qLkw   = 255;  // qLkw
        int vPkw   = 255;  // vPkw
        int vLkw   = 255;  // vLkw
        int tNetto = 255;  // tNetto

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto                                      // tNetto
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ51_8Bit_korrekt_initialisiert() throws Exception {
        int typ  = 51;   // Typ 51 (Option 2, 8 Bit)
        int qKfz = 20;   // qKfz
        int qLkw = 9;    // qLkw
        int vPkw = 155;  // vPkw
        int vLkw = 82;   // vLkw
        int b    = 77;   // b

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) b                                           // b
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(b));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ51_8Bit_mit_fehlerhafter_Belegung_korrekt_initialisiert() throws Exception {
        int typ  = 51;   // Typ 51 (Option 2, 8 Bit)
        int qKfz = 20;   // qKfz
        int qLkw = 9;    // qLkw
        int vPkw = 155;  // vPkw
        int vLkw = 82;   // vLkw
        int b    = 128;  // b > 100%

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) b                                           // b
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-3));  // b > 100%, deshalb auf nicht ermittelbar/fehlerhaft
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ51_8Bit_und_Fehlerwerten_korrekt_initialisiert() throws Exception {
        int typ  = 51;   // Typ 51 (Option 2, 8 Bit)
        int qKfz = 255;  // qKfz
        int qLkw = 255;  // qLkw
        int vPkw = 255;  // vPkw
        int vLkw = 255;  // vLkw
        int b    = 255;  // b

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) b                                           // b
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ52_8Bit_korrekt_initialisiert() throws Exception {
        int typ    = 52;   // Typ 52 (Option 3, 8 Bit)
        int qKfz   = 20;   // qKfz
        int qLkw   = 9;    // qLkw
        int vPkw   = 155;  // vPkw
        int vLkw   = 82;   // vLkw
        int tNetto = 128;  // tNetto
        int b      = 77;   // b
        int sKfz   = 21;   // sKfz
        int vgKfz  = 134;  // vgKfz

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto,                                     // tNetto
            (byte) b,                                          // b
            (byte) sKfz,                                       // sKfz
            (byte) vgKfz                                       // vgKfz
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(tNetto * 100));  // Angabe ist in ms
        assertThat(fg001VerkehrsDaten.getB(), is(b));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(sKfz));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(vgKfz));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ52_8Bit_mit_fehlerhafter_Belegung_korrekt_initialisiert() throws Exception {
        int typ    = 52;   // Typ 52 (Option 3, 8 Bit)
        int qKfz   = 20;   // qKfz
        int qLkw   = 9;    // qLkw
        int vPkw   = 155;  // vPkw
        int vLkw   = 82;   // vLkw
        int tNetto = 128;  // tNetto
        int b      = 128;  // b > 100%
        int sKfz   = 21;   // sKfz
        int vgKfz  = 134;  // vgKfz

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto,                                     // tNetto
            (byte) b,                                          // b
            (byte) sKfz,                                       // sKfz
            (byte) vgKfz                                       // vgKfz
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(tNetto * 100));  // Angabe ist in ms
        assertThat(fg001VerkehrsDaten.getB(), is(-3));                 // b > 100%, deshalb auf nicht ermittelbar/fehlerhaft
        assertThat(fg001VerkehrsDaten.getsKfz(), is(sKfz));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(vgKfz));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ52_8Bit_und_Fehlerwerten_korrekt_initialisiert() throws Exception {
        int typ    = 52;   // Typ 52 (Option 3, 8 Bit)
        int qKfz   = 255;  // qKfz
        int qLkw   = 255;  // qLkw
        int vPkw   = 255;  // vPkw
        int vLkw   = 255;  // vLkw
        int tNetto = 255;  // tNetto
        int b      = 255;  // b
        int sKfz   = 255;  // sKfz
        int vgKfz  = 255;  // vgKfz

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto,                                     // tNetto
            (byte) b,                                          // b
            (byte) sKfz,                                       // sKfz
            (byte) vgKfz                                       // vgKfz
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ53_8Bit_korrekt_initialisiert() throws Exception {
        int typ    = 53;   // Typ 53 (Option 4, 8 Bit)
        int qKfz   = 20;   // qKfz
        int qLkw   = 9;    // qLkw
        int vPkw   = 155;  // vPkw
        int vLkw   = 82;   // vLkw
        int tNetto = 128;  // tNetto
        int b      = 77;   // b
        int sKfz   = 21;   // sKfz
        int vgKfz  = 134;  // vgKfz

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto,                                     // tNetto
            (byte) b,                                          // b
            (byte) sKfz,                                       // sKfz
            (byte) vgKfz                                       // vgKfz
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(tNetto * 100));  // Angabe ist in ms
        assertThat(fg001VerkehrsDaten.getB(), is(b));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(sKfz));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(vgKfz));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ53_8Bit_mit_fehlerhafter_Belegung_korrekt_initialisiert() throws Exception {
        int typ    = 53;   // Typ 53 (Option 4, 8 Bit)
        int qKfz   = 20;   // qKfz
        int qLkw   = 9;    // qLkw
        int vPkw   = 155;  // vPkw
        int vLkw   = 82;   // vLkw
        int tNetto = 128;  // tNetto
        int b      = 128;  // b > 100%
        int sKfz   = 21;   // sKfz
        int vgKfz  = 134;  // vgKfz

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto,                                     // tNetto
            (byte) b,                                          // b
            (byte) sKfz,                                       // sKfz
            (byte) vgKfz                                       // vgKfz
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(qKfz));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(qLkw));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(vPkw));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(vLkw));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(tNetto * 100));  // Angabe ist in ms
        assertThat(fg001VerkehrsDaten.getB(), is(-3));                 // b > 100%, deshalb auf nicht ermittelbar/fehlerhaft
        assertThat(fg001VerkehrsDaten.getsKfz(), is(sKfz));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(vgKfz));
    }

    @Test
    public void test_das_Konstruktor_mit_Typ53_8Bit_und_Fehlerwerten_korrekt_initialisiert() throws Exception {
        int typ    = 53;   // Typ 53 (Option 4, 8 Bit)
        int qKfz   = 255;  // qKfz
        int qLkw   = 255;  // qLkw
        int vPkw   = 255;  // vPkw
        int vLkw   = 255;  // vLkw
        int tNetto = 255;  // tNetto
        int b      = 255;  // b
        int sKfz   = 255;  // sKfz
        int vgKfz  = 255;  // vgKfz

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1,                                          // DE 1
            (byte) typ,                                        // Typ
            (byte) qKfz,                                       // qKfz
            (byte) qLkw,                                       // qLkw
            (byte) vPkw,                                       // vPkw
            (byte) vLkw,                                       // vLkw
            (byte) tNetto,                                     // tNetto
            (byte) b,                                          // b
            (byte) sKfz,                                       // sKfz
            (byte) vgKfz                                       // vgKfz
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test
    public void test_das_Konstruktor_mit_ungueltigem_Typ_korrekt_initialisiert() throws Exception {

        // Initialisierung
        byte[]  tlsBytes = {
            (byte) 1, (byte) 48,                               // DE 1, Typ 48 !!! ungueltiger Typ !!!
            (byte) 20,                                         // qKfz
            (byte) 9,                                          // qLkw
            (byte) 155,                                        // vPkw
            (byte) 82                                          // vLkw
        };
        TlsData tlsData  = new TlsData(tlsBytes);
        DeBlock deBlock  = new DeBlock(tlsData, 12345, 1, 4);  // Knotennummer 12345, FG 1, ID 4)
        String  dePid    = "DE.fuer.JUnit.Tests";

        // Testausfhrung
        Fg001VerkehrsDaten fg001VerkehrsDaten = new Fg001VerkehrsDaten(deBlock, dePid);

        // Erwartetes Ergebnis
        assertThat(fg001VerkehrsDaten.getqKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getqPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getqLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getvPkw(), is(-1));
        assertThat(fg001VerkehrsDaten.getvLkw(), is(-1));
        assertThat(fg001VerkehrsDaten.gettNetto(), is(-1));
        assertThat(fg001VerkehrsDaten.getB(), is(-1));
        assertThat(fg001VerkehrsDaten.getsKfz(), is(-1));
        assertThat(fg001VerkehrsDaten.getVgKfz(), is(-1));
    }

    @Test

    /**
     * Testet die Testflle zur KEx-15, erwartetes Ergebniss sind die Ergebnisse "Soll Formeln"
     * aus der Exceltabelle "PL-Prfung-Kurzzeit-FG1_V1.1_D2015-08-03.xls, Tab "Prfdaten KEx-15 (JUnit)"
     */
    public void test_PlPruefung_und_Korrektur_AFo_DUA_KEx_15() throws Exception {

        // Initialisierung
        //
        // Nachfolgend Mglichkeit, einzelne Testflle temporr auszuschalten..
        Set<Integer> ausgeschlosseneTestfaelle = new HashSet<>();

        // ausgeschlosseneTestfaelle.add(32);
        // ausgeschlosseneTestfaelle.add(36);
        // ausgeschlosseneTestfaelle.add(38);
        // ausgeschlosseneTestfaelle.add(42);
        // ausgeschlosseneTestfaelle.add(43);
        //
        int[][]  eingangsDaten        = {                                                                                                                                                                                              //
            { 12, -1, 4, -1, 86, 73 },                                                                                                                                                                                                 // Fall 1: int qKfz, int qPkw, int qLkw, int vKfz, int vPkw, int vLkw
            { 12, -1, -1, -1, 86, 73 },                                                                                                                                                                                                // Fall 2
            { 12, -1, 4, -1, -1, 73 },                                                                                                                                                                                                 // Fall 3
            { 12, -1, 4, -1, 86, -1 },                                                                                                                                                                                                 // Fall 4
            { 12, -1, 4, -1, -1, -1 },                                                                                                                                                                                                 // Fall 5
            { 12, -1, -1, -1, -1, 73 },                                                                                                                                                                                                // Fall 6
            { 12, -1, -1, -1, 86, -1 },                                                                                                                                                                                                // Fall 7
            { 12, -1, -1, -1, -1, -1 },                                                                                                                                                                                                // Fall 8
            { -1, -1, 4, -1, 86, 73 },                                                                                                                                                                                                 // Fall 9
            { -1, -1, -1, -1, 86, 73 },                                                                                                                                                                                                // Fall 10
            { -1, -1, 4, -1, -1, 73 },                                                                                                                                                                                                 // Fall 11
            { -1, -1, 4, -1, 86, -1 },                                                                                                                                                                                                 // Fall 12
            { -1, -1, 4, -1, -1, -1 },                                                                                                                                                                                                 // Fall 13
            { -1, -1, -1, -1, -1, 73 },                                                                                                                                                                                                // Fall 14
            { -1, -1, -1, -1, 86, -1 },                                                                                                                                                                                                // Fall 15
            { -1, -1, -1, -1, -1, -1 },                                                                                                                                                                                                // Fall 16
            { 9, -1, 14, -1, 82, 75 },                                                                                                                                                                                                 // Fall 17
            { 9, -1, 14, -1, -1, 75 },                                                                                                                                                                                                 // Fall 18
            { 9, -1, 14, -1, 82, -1 },                                                                                                                                                                                                 // Fall 19
            { 9, -1, 14, -1, -1, -1 },                                                                                                                                                                                                 // Fall 20
            { 0, -1, 6, -1, -1, 78 },                                                                                                                                                                                                  // Fall 21
            { 0, -1, 6, -1, 84, -1 },                                                                                                                                                                                                  // Fall 22
            { 0, -1, 6, -1, -1, -1 },                                                                                                                                                                                                  // Fall 23
            { 0, -1, 6, -1, 84, 78 },                                                                                                                                                                                                  // Fall 24
            { 0, -1, -1, -1, 84, 78 },                                                                                                                                                                                                 // Fall 25
            { 0, -1, -1, -1, 84, -1 },                                                                                                                                                                                                 // Fall 26
            { 0, -1, -1, -1, -1, 78 },                                                                                                                                                                                                 // Fall 27
            { 0, -1, -1, -1, -1, -1 },                                                                                                                                                                                                 // Fall 28
            { 10, -1, 0, -1, 90, -1 },                                                                                                                                                                                                 // Fall 29
            { 10, -1, 0, -1, -1, 77 },                                                                                                                                                                                                 // Fall 30
            { 10, -1, 0, -1, -1, -1 },                                                                                                                                                                                                 // Fall 31
            { 10, -1, 0, -1, 90, 77 },                                                                                                                                                                                                 // Fall 32
            { -1, -1, 0, -1, 90, 77 },                                                                                                                                                                                                 // Fall 33
            { -1, -1, 0, -1, 90, -1 },                                                                                                                                                                                                 // Fall 34
            { -1, -1, 0, -1, -1, 77 },                                                                                                                                                                                                 // Fall 35
            { -1, -1, 0, -1, -1, -1 },                                                                                                                                                                                                 // Fall 36
            { 8, -1, 8, -1, -1, 80 },                                                                                                                                                                                                  // Fall 37
            { 8, -1, 8, -1, 95, 80 },                                                                                                                                                                                                  // Fall 38
            { 8, -1, 8, -1, 95, -1 },                                                                                                                                                                                                  // Fall 39
            { 8, -1, 8, -1, -1, -1 },                                                                                                                                                                                                  // Fall 40
            { 0, -1, 0, -1, -1, -1 },                                                                                                                                                                                                  // Fall 41
            { 0, -1, 0, -1, 99, -1 },                                                                                                                                                                                                  // Fall 42
            { 0, -1, 0, -1, -1, 79 },                                                                                                                                                                                                  // Fall 43
            { 0, -1, 0, -1, 99, 79 },                                                                                                                                                                                                  // Fall 44
            { 3, -1, 2, -1, 0, 7 },                                                                                                                                                                                                    // Fall 45
            { 3, -1, 1, -1, 6, 0 },                                                                                                                                                                                                    // Fall 46
            { 3, -1, 2, -1, 0, 0 },                                                                                                                                                                                                    // Fall 47
            { 3, -1, 1, -1, 0, 0 }                                                                                                                                                                                                     // Fall 48
        };
        int[][]  sollDaten            = {
            { 12, 8, 4, 82, 86, 73 },                                                                                                                                                                                                  // Fall 1: int qKfz, int qPkw, int qLkw, int vKfz, int vPkw, int vLkw
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 2
            { 12, 8, 4, -3, -3, -3 },                                                                                                                                                                                                  // Fall 3
            { 12, 8, 4, -3, -3, -3 },                                                                                                                                                                                                  // Fall 4
            { 12, 8, 4, -3, -3, -3 },                                                                                                                                                                                                  // Fall 5
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 6
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 7
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 8
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 9
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 10
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 11
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 12
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 13
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 14
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 15
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 16
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 17
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 18
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 19
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 20
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 21
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 22
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 23
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 24
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 25
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 26
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 27
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 28
            { 10, 10, 0, 90, 90, -1 },                                                                                                                                                                                                 // Fall 29
            { 10, 10, 0, -3, -3, -3 },                                                                                                                                                                                                 // Fall 30
            { 10, 10, 0, -1, -1, -1 },                                                                                                                                                                                                 // Fall 31
            { 10, 10, 0, 90, 90, 77 },                                                                                                                                                                                                 // Fall 32
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 33
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 34
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 35
            { -3, -3, -3, -3, -3, -3 },                                                                                                                                                                                                // Fall 36
            { 8, 0, 8, 80, -1, 80 },                                                                                                                                                                                                   // Fall 37
            { 8, 0, 8, 80, 95, 80 },                                                                                                                                                                                                   // Fall 38
            { 8, 0, 8, -3, -3, -3 },                                                                                                                                                                                                   // Fall 39
            { 8, 0, 8, -1, -1, -1 },                                                                                                                                                                                                   // Fall 40
            { 0, 0, 0, -1, -1, -1 },                                                                                                                                                                                                   // Fall 41
            { 0, 0, 0, 99, 99, -1 },                                                                                                                                                                                                   // Fall 42
            { 0, 0, 0, 79, -1, 79 },                                                                                                                                                                                                   // Fall 43
            { 0, 0, 0, -3, -3, -3 },                                                                                                                                                                                                   // Fall 44
            { 3, 1, 2, 5, 0, 7 },                                                                                                                                                                                                      // Fall 45
            { 3, 2, 1, 4, 6, 0 },                                                                                                                                                                                                      // Fall 46
            { 3, 1, 2, 0, 0, 0 },                                                                                                                                                                                                      // Fall 47
            { 3, 2, 1, 0, 0, 0 }                                                                                                                                                                                                       // Fall 48
        };
        String[] betriebsMeldungsText = {
            "",                                                                                                                                                                                                                        // Fall 1
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=12 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=86 und vLkw=73.",  // Fall 2
            "Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=12, qPkw=8, qLkw=4, vPkw=-1 und vLkw=73.",                                                                                                  // Fall 3
            "Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=12, qPkw=8, qLkw=4, vPkw=86 und vLkw=-1.",                                                                                                  // Fall 4
            "Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=12, qPkw=8, qLkw=4, vPkw=-1 und vLkw=-1.",                                                                                                  // Fall 5
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=12 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=73.",  // Fall 6
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=12 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=86 und vLkw=-1.",  // Fall 7
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=12 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=-1.",  // Fall 8
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-1 und qLkw=4. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=86 und vLkw=73.",   // Fall 9
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-1 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=86 und vLkw=73.",  // Fall 10
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-1 und qLkw=4. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=73.",   // Fall 11
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-1 und qLkw=4. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=86 und vLkw=-1.",   // Fall 12
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-1 und qLkw=4. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=-1.",   // Fall 13
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-1 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=73.",  // Fall 14
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-1 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=86 und vLkw=-1.",  // Fall 15
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-1 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=-1.",  // Fall 16
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=9 und qLkw=14. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=82 und vLkw=75.",   // Fall 17
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=9 und qLkw=14. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=75.",   // Fall 18
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=9 und qLkw=14. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=82 und vLkw=-1.",   // Fall 19
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=9 und qLkw=14. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=-1.",   // Fall 20
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=0 und qLkw=6. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=78.",    // Fall 21
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=0 und qLkw=6. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=84 und vLkw=-1.",    // Fall 22
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=0 und qLkw=6. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=-1.",    // Fall 23
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=0 und qLkw=6. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=84 und vLkw=78.",    // Fall 24
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=0 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=84 und vLkw=78.",   // Fall 25
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=0 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=84 und vLkw=-1.",   // Fall 26
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=0 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=78.",   // Fall 27
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=0 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=-1.",   // Fall 28
            "",                                                                                                                                                                                                                        // Fall 29
            "Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=10, qPkw=10, qLkw=0, vPkw=-1 und vLkw=77.",                                                                                                 // Fall 30
            "",                                                                                                                                                                                                                        // Fall 31
            "",                                                                                                                                                                                                                        // Fall 32
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-1 und qLkw=0. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=90 und vLkw=77.",   // Fall 33
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-1 und qLkw=0. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=90 und vLkw=-1.",   // Fall 34
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-1 und qLkw=0. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=77.",   // Fall 35
            "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-1 und qLkw=0. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=-1.",   // Fall 36
            "",                                                                                                                                                                                                                        // Fall 37
            "",                                                                                                                                                                                                                        // Fall 38
            "Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=8, qPkw=0, qLkw=8, vPkw=95 und vLkw=-1.",                                                                                                   // Fall 39
            "",                                                                                                                                                                                                                        // Fall 40
            "",                                                                                                                                                                                                                        // Fall 41
            "",                                                                                                                                                                                                                        // Fall 42
            "",                                                                                                                                                                                                                        // Fall 43
            "Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=0, qPkw=0, qLkw=0, vPkw=99 und vLkw=79.",                                                                                                   // Fall 44
            "",                                                                                                                                                                                                                        // Fall 45
            "",                                                                                                                                                                                                                        // Fall 46
            "",                                                                                                                                                                                                                        // Fall 47
            ""                                                                                                                                                                                                                         // Fall 48
        };

        for (int testfall = 0; testfall < eingangsDaten.length; testfall++) {
            if (ausgeschlosseneTestfaelle.contains(testfall + 1)) {
                continue;
            }

            // Initialisierung
            Fg001VerkehrsDaten fg001VerkehrsDatenEingang = new Fg001VerkehrsDaten(eingangsDaten[testfall][0], eingangsDaten[testfall][1], eingangsDaten[testfall][2], eingangsDaten[testfall][3], eingangsDaten[testfall][4], eingangsDaten[testfall][5]);

            // Fg001VerkehrsDaten fg001VerkehrsDatenSoll = new Fg001VerkehrsDaten(eingangsDaten[testfall][0], eingangsDaten[testfall][1], eingangsDaten[testfall][2], eingangsDaten[testfall][3], eingangsDaten[testfall][4], eingangsDaten[testfall][5]);
            Fg001VerkehrsDaten fg001VerkehrsDatenSoll = new Fg001VerkehrsDaten(sollDaten[testfall][0], sollDaten[testfall][1], sollDaten[testfall][2], sollDaten[testfall][3], sollDaten[testfall][4], sollDaten[testfall][5]);

            // Testausfhrung
            fg001VerkehrsDatenEingang.plPruefungUndKorrektur();

            // Erwartetes Ergebnis
            assertThat("Testfall " + (testfall + 1), fg001VerkehrsDatenEingang, is(fg001VerkehrsDatenSoll));

            // Erwartets Ergebnis Betriebsmeldungstext
            assertThat("Testfall " + (testfall + 1), fg001VerkehrsDatenEingang.getBetriebsMeldung(), is(betriebsMeldungsText[testfall]));

            // System.out.format("\"%s\"%n", fg001VerkehrsDatenEingang.getBetriebsMeldung());
        }
    }

    @Test
    public void test_das_Betriebsmeldungen_korrekt_am_DaV_publiziert_werden() throws Exception {

        // Initialisierung
        Debug debug              = Debug.getLogger();
        String[]            dafAufrufParameter = { "-benutzer=inovat", "-authentifizierung=passwd" };
        ClientDavParameters clientDavParameter = new ClientDavParameters(dafAufrufParameter);
        ClientDavInterface daf                = new ClientDavConnection(clientDavParameter);

        daf.connect();
        daf.login();

        MessageSender messageSender = MessageSender.getInstance();

        messageSender.init(daf, "KExTlsOsi7Test", "");

        String            betriebsMeldung   = "Attribute q nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=0 und qLkw=-1. Attribute v nicht ermittelbar/fehlerhaft an DE [DE.fuer.JUnit.Tests], da qKfz=-3, qPkw=-3, qLkw=-3, vPkw=-1 und vLkw=-1.";

        // Testausfhrung
        try {
            if (!betriebsMeldung.equals("")) {
                messageSender.sendMessage("KEx-BD-VW01", MessageType.APPLICATION_DOMAIN, "", MessageGrade.INFORMATION, null, MessageState.MESSAGE, betriebsMeldung);
                messageSender.sendMessage("KEx-BD-VW01", MessageType.APPLICATION_DOMAIN, "", MessageGrade.INFORMATION, null, MessageState.MESSAGE, betriebsMeldung);
            }
        }
        catch (Exception e) {
            debug.warning("Betriebsmeldung konnte nicht verschickt werden" + betriebsMeldung, e);
        }

        // Erwartetes Ergebnis
    }


}


//~Formatiert mit 'inovat Kodierkonvention' am 05.08.15
