/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.Fg003Default
 *
 * de.bsvrz.kex.tls.osi7.conversion.Fg003Default is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg003Default is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.Fg003Default.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.Fg003Default.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg003Default ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg003Default wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */



package de.bsvrz.kex.tls.osi7.conversion;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.tls.osi7.De;
import de.bsvrz.kex.tls.osi7.deausfallueberwachung.DeAusfallUeberwachung;
import de.bsvrz.kex.tls.osi7.DeBlock;
import de.bsvrz.kex.tls.osi7.Eak;
import de.bsvrz.kex.tls.osi7.IllegalOsi7TelegramStructureException;
import de.bsvrz.kex.tls.osi7.TlsApplicationLayer;
import de.bsvrz.kex.tls.osi7.TlsData;
import de.bsvrz.kex.tls.TlsApplication;
import de.bsvrz.sys.funclib.debug.Debug;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Umsetungsmodul fr die FG 3, Standard (alle Typen der FG 3 gem TLS 2002). Angemeldet werden aber nur die
 * Telegrammtypen, die bei allen FG 3 DE identisch sind, also DeFehler, Kanalsteuerung etc. Umsetzungsmodul enthlt zwar
 * alle Umsetzungsmethoden, fr einen konkreten FG 3 Typ (z.B. Helligkeit) ist aber fr dieses De in der Konfiguration
 * als Umsetzungsklasse aber die spezielle, von dieser Klasse abgeleitete Klasse (bei Helligkeit {@link Fg003Typ061} )
 * zu verwenden, in der dann die spezifischen Umsetzungsmethoden angemeldet werden. Bei FG 3 De ist Angabe der
 * speziellen Umsetzungsklasse in der Konfiguration obligatorisch, es wird keine Umsetzungklasse bei Fehlen einer
 * solchen Angabe standardmig verwendet!
 * <p/>
 * Erweiterung: untersttzt auch nachgelieferte Telegramme (Standard ID+32)
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class Fg003Default extends FgAllDefault {

    // ------------------------------------------------------------------------
    // VARIABLENDEKLARATIONEN
    // ------------------------------------------------------------------------
    // Sonstige ---------------------------------------------------------------

    /** DebugLogger fr Debug-Ausgaben */
    private static final Debug debug = Debug.getLogger();

    //~ FELDER ================================================================

    /**
     * Intervalldauer UFD aus den Betriebsparametern, wird u.a. fuer die DeAusfallUeberwachung verwendet. Voreinstellung
     * ist der Wert aus dem Aufrufparameter -standardPeriodenDauerUfdFuerDatenAusfallErkennungInSekunden
     */
    int _intervallDauerUFD = TlsApplication.getStandardPeriodenDauerUfdFuerDatenAusfallErkennungInSekunden();

    /**
     * Tabelle, in der das Verhalten beim Empfang von TLS De-Blcken eingetragen wird. Jede Tabellenzeile hat folgenden
     * Aufbau:
     * <pre>
     * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
     * z.B.
     * "atg.tlsGloKanalSteuerung", "asp.tlsAntwort", "1", "29", "2", "typ.deUfd"
     * </pre>
     * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Fr jede Attributguppen/Aspekt Kombination erfolgt eine
     * Sendeanmeldung als Quelle.</li> <li> In der Methode {@link #processDeBlock} wird fr jede zum De-Block passende
     * Fg/Id/Typ-Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag wie folgt gebildet wird (hier
     * fr den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspTlsAntwort(DeBlock, Data,
     * DataDescription)</code><br> Dabei enthlt der <code>deBlock</code> die Daten des zu verarbeitenden De-Blocks und
     * <code>data</code> ein Datenobjekt der passenden Attributgruppen/Aspekt-Kombination, welchem direkt die Inhalte
     * des De-Blocks zugewiesen werden knnen.<code>DataDescription</code> enthlt die passende {@link DataDescription}
     * fr die zu sendenden Daten.</li> <li> Fr jede Attributguppen/Aspekt Kombination erfolgt eine Sendeanmeldung als
     * Quelle. Sind mehrere passende Eintrage vorhanden, werden entsprechend mehrere Methodenaufrufe durchgefhrt.
     * Dadurch kann ein De-Block mehrfach ausgewert und an verschiedene Attributgruppen weitergerleitet werden (z.B.
     * protokollnahe TLS-Attributgruppe und Attributgruppe eine Fachmodells):</li> <li> Ein Eintrag von "255" entspricht
     * dabei einer "Wildcard" (passt immer). </ul>
     */
    private final Object[][] tlsToDav = new Object[][] {
        { "atg.tlsGloDeFehler", "asp.tlsAntwort", "3", "1", "1", "typ.deUfd" },                     //
        { "atg.tlsGloDeFehler", "asp.tlsAntwort", "3", "1", "33", "typ.deUfd" },                    //
        { "atg.tlsGloKanalSteuerung", "asp.parameterIst", "255", "29", "2", "typ.deUfd" },          //
        { "atg.tlsGloKanalSteuerung", "asp.parameterIst", "255", "29", "34", "typ.deUfd" },         //
        { "atg.tlsGloGeographischeKenndaten", "asp.tlsZustand", "255", "36", "3", "typ.deUfd" },    //
        { "atg.tlsGloGeographischeKenndaten", "asp.tlsZustand", "255", "36", "35", "typ.deUfd" },   //
        { "atg.tlsUfdErgnzendeDeFehlerMeldung", "asp.tlsAntwort", "3", "14", "1", "typ.deUfd" },   //
        { "atg.tlsUfdErgnzendeDeFehlerMeldung", "asp.tlsAntwort", "3", "14", "33", "typ.deUfd" },  //
        { "atg.tlsUfdNegativeQuittung", "asp.tlsAntwort", "3", "16", "2", "typ.deUfd" },            //
        { "atg.tlsUfdNegativeQuittung", "asp.tlsAntwort", "3", "16", "34", "typ.deUfd" },           //
        { "atg.tlsUfdBetriebsParameter", "asp.parameterIst", "3", "32", "3", "typ.deUfd" },         //
        { "atg.tlsUfdBetriebsParameter", "asp.parameterIst", "3", "32", "35", "typ.deUfd" }
    };                                                                                              //

    // Private ----------------------------------------------------------------

    /**
     * Tabelle, in der das Verhalten beim Empfang von Attributgruppen eingetragen wird. Jede Tabellenzeile hat folgenden
     * Aufbau:
     * <pre>
     * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
     * z.B.
     * "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "1", "29", "2", "typ.deUfd"
     * </pre>
     * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Fr jede Attributguppen/Aspekt Kombination erfolgt eine
     * Empfangsanmeldung als Empfnger.</li> <li> In der Methode {@link #update} wird fr jede zur
     * Attributgruppen/Aspket-Kombination passende Kombination eine Methode aufgerufen, deren Namen aus dem
     * Tabelleneintrag wie folgt gebildet wird (hier fr den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll(ResultData)</code><br>
     * Dabei enthlt <code>ResultData</code> die Daten der zu verarbeitenden ATG/ASP-Kombination.</li> <li> Fr jede
     * Attributguppen/Aspekt Kombination erfolgt eine Empfangsanmeldung als Empfnger. Sind mehrere passende Eintrage
     * vorhanden, werden entsprechend mehrere Methodenaufrufe durchgefhrt. Dadurch knnen mehrere ATG/ASP-Kombinationen
     * in einen De-Block abgebildet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eines
     * Fachmodells):</li> <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
     */
    private final Object[][] davToTls = new Object[][] {

        // Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
        { "atg.tlsGloDeBlockAbruf", "asp.tlsAbruf", "255", "255", "255", "typ.deUfd" }, { "atg.tlsGloDeBlockSenden", "asp.tlsVorgabe", "255", "255", "255", "typ.deUfd" }, { "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "255", "29", "2", "typ.deUfd" }, { "atg.tlsGloGeographischeKenndaten", "asp.tlsVorgabe", "255", "36", "3", "typ.deUfd" }, { "atg.tlsUfdBetriebsParameter", "asp.parameterSoll", "3", "32", "3", "typ.deUfd" }
    };

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    // ------------------------------------------------------------------------
    // KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
    // ------------------------------------------------------------------------

    /**
     * Erzeugt ein Objekt vom Typ Fg003Default
     *
     * @param daf Datenverteilerverbindung
     * @param eak Eak.
     * @param de  De.
     */
    public Fg003Default(ClientDavInterface daf, Eak eak, De de) {
        super(daf, eak, de);
        subscribeAllData(tlsToDav, davToTls);
    }

    //~ METHODEN ==============================================================

    /**
     * FG 3 : Typ: 1 Id: 1 Name: DE-Fehler (Antwort). Ergnzt globale Behandlung De-Fehler in {@link
     * #Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort}. Beim Empfang einer De-Gutmeldung wird die De initialisiert,
     * indem <ul> <li>Ufd-kanalsteuerung {@link #Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll}</li>
     * <li>Ufd-betriebsparameter{@link #Fg003Typ032Id003AtgTlsUfdBetriebsParameterAspParameterSoll}</li> <ul> versendet
     * werden.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData        resultData     = super.Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);
        IntegerValueState DEFehlerStatus = data.getUnscaledValue("DEFehlerStatus").getState();

        if (DEFehlerStatus != null) {
            debug.fine("DEFehlerStatus: " + DEFehlerStatus.getName());

            if (DEFehlerStatus.getName().equals("ok")) {
                sendInitialDeBlock(deBlock.getKey(), 29, 2);  // UFD-Kanalsteuerung senden
                sendInitialDeBlock(deBlock.getKey(), 32, 3);  // UFD-Betriebsparameter senden
            } else {
                List deList = new ArrayList();

                deList.add(_de);
                TlsApplicationLayer.writeNoDataToUnreachableDe(deList, _eakConversionModul.getTimeStamp(), -1, "atg.tlsGloDeFehler");
            }
        }

        return resultData;
    }

    // ------------------------------------------------------------------------
    // SONSTIGE METHODEN
    // ------------------------------------------------------------------------

    /**
     * Entspricht {@link de.bsvrz.kex.tls.osi7.conversion.Fg003Default#Fg003Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock,
     * Data, DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ001Id033AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 1, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg003Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 3 : Typ: 14 Id: 1 Name: Ergnzende DE-Fehlermeldung (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          [ 9..29 ]
     * Byte 2           Daten-Endgerte-Kanal                   [ 1..254 ]
     * Byte 3           Typ der DE-Daten                        [ 14 ]
     * Byte 4           Herstellercode                          (s. Anh. 7, Kap. 2)
     * Byte 5           Anzahl TLS  Fehlerbytes                [ 4 ]
     * Byte 6           TLS - Fehlerbyte 1                      Codierung siehe unten
     * Byte 7           TLS - Fehlerbyte 2                      Codierung siehe unten
     * Byte 8           TLS - Fehlerbyte 3                      Codierung siehe unten
     * Byte 9           TLS - Fehlerbyte 4                      Codierung siehe unten
     * Byte 10          Anzahl Hersteller Fehlerbytes           [ 0..20 ]
     * Byte 11          Hersteller-Fehlerbyte 1                 herstellerspez. Codierung
     * Byte..           Hersteller-Fehlerbyte n
     * </pre>
     * Inhalt TLS-Fehlerbytes:<br> Die Fehlerbytes sind bitweise codiert. Jedes Bit definiert einen bestimmten Fehler,
     * wobei das Bit gleich 1 gesetzt ist.<br> Es knnen auch mehrere Bit gleichzeitig gesetzt werden. Der Inhalt der
     * Fehlerbytes ist optional. D.h. bei einer Realisierung dieses Telegrammtyps mssen vom Hersteller nicht alle in
     * der Tabelle definierten Fehlercodes erfllt werden. Fr sptere Definitionen reservierte Bits bzw. nicht
     * verwendete Bits sind gleich Null zu setzen.
     * <pre>
     *         Bit 7      Bit 6       Bit 5       Bit 4          Bit 3         Bit 2       Bit 1       Bit 0
     * Byte 1  Reserve    Reserve     RNR am      Lokalbus       Reserve       Reserve     Reserve     Teilstrung
     *                                Lokalbus1   ausgefallen
     * Byte 2  Reserve    Reserve     Reserve     Reserve        Rohwert       Sensor      RNR am      Subbus
     *                                                           auerhalb     defekt      Subbus      ausgefallen
     *                                                           der Grenzen
     * Byte 3  Reserve    Reserve     Reserve     Reserve        Reserve       Reserve     Reserve     Reserve
     * Byte 4  Reserve    Reserve     Reserve     Reserve        Reserve       Reserve     Reserve     Reserve
     * </pre>
     * 1) nur in Verbindung mit St/SM=1 im Byte 4 der DE-Block-Struktur im Typ 1<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ014Id001AtgTlsUfdErgnzendeDeFehlerMeldungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Herstellercode-Byte aus Telegramm ---------------------------------------------------------------------------
        int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Hersteller").set(_Herstellercode_);

        // Fehler-Bytes aus Telegramm -----------------------------------------------------------------------------------
        deBlock.getDeData().readUnsignedByteAsInt();  // Anzahl der folgenden Bytes berlesen, da immer == 4 !

        int              _TlsFehler1_ = deBlock.getDeData().readUnsignedByteAsInt();
        int              _TlsFehler2_ = deBlock.getDeData().readUnsignedByteAsInt();
        int              _TlsFehler3_ = deBlock.getDeData().readUnsignedByteAsInt();
        int              _TlsFehler4_ = deBlock.getDeData().readUnsignedByteAsInt();
        Data.NumberArray _TlsFehler_  = data.getItem("FehlercodesTLS").getUnscaledArray("Fehler");

        _TlsFehler_.setLength(4);
        _TlsFehler_.getValue(0).set(_TlsFehler1_);
        _TlsFehler_.getValue(1).set(_TlsFehler2_);
        _TlsFehler_.getValue(2).set(_TlsFehler3_);
        _TlsFehler_.getValue(3).set(_TlsFehler4_);

        // Hersteller Fehler-Bytes aus Telegramm -----------------------------------------------------------------------
        int   count              = deBlock.getDeData().readUnsignedByteAsInt();
        int[] _HerstellerFehler_ = new int[count];

        for (int i = 0; i < count; i++) {
            _HerstellerFehler_[i] = deBlock.getDeData().readUnsignedByteAsInt();
        }

        Data.NumberArray _Fehler_ = data.getItem("FehlercodesHersteller").getUnscaledArray("Fehler");

        _Fehler_.setLength(count);

        for (int i = 0; i < count; i++) {
            _Fehler_.getValue(i).set(_HerstellerFehler_[i]);
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * Entspricht {@link de.bsvrz.kex.tls.osi7.conversion.Fg003Default#Fg003Typ014Id001AtgTlsUfdErgnzendeDeFehlerMeldungAspTlsAntwort(DeBlock,
     * Data, DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ014Id033AtgTlsUfdErgnzendeDeFehlerMeldungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 1, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg003Typ014Id001AtgTlsUfdErgnzendeDeFehlerMeldungAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 3 : Typ: 16 Id: 2 Name: Negative Quittung (Antwort) TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          Lnge des folgenden DE-Blocks [ 4 ]
     * Byte 2           Daten-Endgerte-Kanal                   [ 1..254 ]
     * Byte 3           Typ der DE-Daten                        [ 16 ]
     * Byte 4           Fehlerursache                           siehe unten
     * Byte 5           Herstellercode                          siehe Liste in Anhang 7, Kap. 2
     * </pre>
     * Inhalt Byte: Fehlerursache unter FG 3:
     * <pre>
     *        0:   sonstige Fehlerursache
     *        1:   unbekannte oder nicht auswertbare ID
     *        2:   unbekannter oder nicht auswertbarer Typ
     *
     *   3..127:   reserviert fr sptere Definitionen
     * 128..255:   frei fr herstellerdefinierte Ursachencodes
     *
     * </pre>
     * Inhalt Byte Herstellercode<br> Der Herstellercode ist ein fr jeden Hersteller eindeutig zugeordneter Code, der
     * der Zentrale die Zuordnung der Nachrichten erleichtern soll.<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ016Id002AtgTlsUfdNegativeQuittungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        return super.Fg255Typ016Id002AtgTlsGloNegativeQuittungAspTlsAntwort(deBlock, data, dataDescription);
    }

    /**
     * Entspricht {@link de.bsvrz.kex.tls.osi7.conversion.Fg003Default#Fg003Typ016Id002AtgTlsUfdNegativeQuittungAspTlsAntwort(DeBlock,
     * Data, DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ016Id034AtgTlsUfdNegativeQuittungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg003Typ016Id002AtgTlsUfdNegativeQuittungAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 3 : Typ: 32 Id: 3 Name: UFD-Betriebsparameter (Abruf / Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          Lnge des folgenden DE-Blocks [ 5 ]
     * Byte 2           Daten-Endgerte-Kanal                   [ 1.. 254, 255 ,(evt. Clusterkanal)1 ]
     * Byte 3           Typ der DE-Daten                        [ 32 ]
     * Byte 4           Erfassungsperiodendauer                 [ 0..65535 s ] low Byte
     * Byte 5           Erfassungsperiodendauer                 high Byte
     * Byte 6           bertragungsverfahren                   siehe unten
     * 1) im Anwendungsfall Clusterkanal gilt entsprechend die Definition Seite A6.2.6 - 8, Kapitel 6.2.7
     * </pre>
     * Inhalt Byte Erfassungsperiodendauer<br> Der Wert "Erfassungsperiodendauer" gibt an, in welchen zeitlichen
     * Abstnden die bertragung des Messergebnisses erfolgen soll. Es sind nur Werte zugelassen, die ein ganzzahliges
     * Vielfaches haben, welches 24 Stunden ergibt (1, 2, 3, 4, 5, 6, 10, 12, 15, 20, und 30 Minuten und 1, 1, 2, 3, 4,
     * 6, 8 und 12 Stunden).<br> Davon zu unterscheiden ist die Dauer der Messperiode. Die Messperiode ist sensor- und
     * herstellerspezifisch. Beispielsweise liegt dem Mittelwert fr die Windgeschwindigkeit eine bestimmte Messperiode
     * zugrunde.<br> Der Wert "Erfassungsperiodendauer" ist nur relevant bei der zyklischen Abgabe von Meldungen (Byte
     * "bertragungsverfahren" = 1).<br> <br> Inhalt Byte bertragungsverfahren<br> Das Byte gibt Anweisung, wann die
     * Daten der Messwertgeber zur Zentrale bertragen werden: Entweder auf Abruf durch die Zentrale (Ereignisklasse 2),
     * spontan nach Ende der o.g. Erfassungsperiode (Ereignisklasse 1). Meldung nur nach Abruf durch die Zentrale
     * bedeutet, dass der DE ohne Aufforderung keine Meldungen mehr abgibt.
     * <pre>
     *         0: Meldung nur nach Abruf
     *         1: Zyklische Abgabe von Meldungen
     *    2..255: nicht definiert
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ032Id003AtgTlsUfdBetriebsParameterAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Erfassungsperiodendauer_-Wert aus Telegramm ------------------------------------------------------------------
        int _Erfassungsperiodendauer_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        switch (_Erfassungsperiodendauer_) {
        case 60 :     // 1 Minute
        case 120 :    // 2 Minuten
        case 180 :    // 3 Minuten
        case 240 :    // 4 Minuten
        case 300 :    // 5 Minuten
        case 360 :    // 6 Minuten
        case 600 :    // 10 Minuten
        case 720 :    // 12 Minuten
        case 900 :    // 15 Minuten
        case 1200 :   // 20 Minuten
        case 1800 :   // 30 Minuten
        case 3600 :   // 1 Stunde
        case 5400 :   // 1 Stunden
        case 7200 :   // 2 Stunden
        case 10800 :  // 3 Stunden
        case 14400 :  // 4 Stunden
        case 21600 :  // 6 Stunden
        case 28800 :  // 8 Stunden
        case 43200 :  // 12 Stunden
            data.getUnscaledValue("Erfassungsperiodendauer").set(_Erfassungsperiodendauer_);

            // Aktuellen Wert fuer externen Zugriff merken
            _intervallDauerUFD = _Erfassungsperiodendauer_;

            break;

        default :
            debug.warning("_Erfassungsperiodendauer_ ungltig : " + _Erfassungsperiodendauer_ + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());

            return resultData;
        }

        // bertragungsverfahren-Byte aus Telegramm --------------------------------------------------------------------
        int _bertragungsverfahren_ = deBlock.getDeData().readUnsignedByteAsInt();

        switch (_bertragungsverfahren_) {
        case 0 :
        case 1 :
            data.getUnscaledValue("bertragungsverfahren").set(_bertragungsverfahren_);

            break;

        default :
            debug.warning("bertragungsverfahren ungltig : " + _bertragungsverfahren_ + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());

            return resultData;
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 32 Id: 3 Name: UFD-Betriebsparameter (Abruf / Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          Lnge des folgenden DE-Blocks [ 5 ]
     * Byte 2           Daten-Endgerte-Kanal                   [ 1.. 254, 255 ,(evt. Clusterkanal)1 ]
     * Byte 3           Typ der DE-Daten                        [ 32 ]
     * Byte 4           Erfassungsperiodendauer                 [ 0..65535 s ] low Byte
     * Byte 5           Erfassungsperiodendauer                 high Byte
     * Byte 6           bertragungsverfahren                   siehe unten
     * 1) im Anwendungsfall Clusterkanal gilt entsprechend die Definition Seite A6.2.6 - 8, Kapitel 6.2.7
     * </pre>
     * Inhalt Byte Erfassungsperiodendauer<br> Der Wert "Erfassungsperiodendauer" gibt an, in welchen zeitlichen
     * Abstnden die bertragung des Messergebnisses erfolgen soll. Es sind nur Werte zugelassen, die ein ganzzahliges
     * Vielfaches haben, welches 24 Stunden ergibt (1, 2, 3, 4, 5, 6, 10, 12, 15, 20, und 30 Minuten und 1, 1, 2, 3, 4,
     * 6, 8 und 12 Stunden).<br> Davon zu unterscheiden ist die Dauer der Messperiode. Die Messperiode ist sensor- und
     * herstellerspezifisch. Beispielsweise liegt dem Mittelwert fr die Windgeschwindigkeit eine bestimmte Messperiode
     * zugrunde.<br> Der Wert "Erfassungsperiodendauer" ist nur relevant bei der zyklischen Abgabe von Meldungen (Byte
     * "bertragungsverfahren" = 1).<br> <br> Inhalt Byte bertragungsverfahren<br> Das Byte gibt Anweisung, wann die
     * Daten der Messwertgeber zur Zentrale bertragen werden: Entweder auf Abruf durch die Zentrale (Ereignisklasse 2),
     * spontan nach Ende der o.g. Erfassungsperiode (Ereignisklasse 1). Meldung nur nach Abruf durch die Zentrale
     * bedeutet, dass der DE ohne Aufforderung keine Meldungen mehr abgibt.
     * <pre>
     *         0: Meldung nur nach Abruf
     *         1: Zyklische Abgabe von Meldungen
     *    2..255: nicht definiert
     * </pre>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg003Typ032Id003AtgTlsUfdBetriebsParameterAspParameterSoll(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int _Erfassungsperiodendauer_;
        int _bertragungsverfahren_;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _Erfassungsperiodendauer_ = data.getUnscaledValue("Erfassungsperiodendauer").intValue();
        _bertragungsverfahren_   = data.getUnscaledValue("bertragungsverfahren").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(3);

        deApplicationData.writeIntAsLowHighByte(_Erfassungsperiodendauer_);
        deApplicationData.writeIntAsByte(_bertragungsverfahren_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 32, 3, _de.getDeChannel());
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        // Aktuellen Wert fuer externen Zugriff merken
        _intervallDauerUFD = _Erfassungsperiodendauer_;

        return deBlock;
    }

    /**
     * Entspricht {@link de.bsvrz.kex.tls.osi7.conversion.Fg003Default#Fg003Typ032Id003AtgTlsUfdBetriebsParameterAspParameterIst(DeBlock,
     * Data, DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ032Id035AtgTlsUfdBetriebsParameterAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg003Typ032Id003AtgTlsUfdBetriebsParameterAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 3 : Typ: 35 Id: 4 Name: UFD-xyz (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ035Id004AtgTlsUfdErgebnisMeldungTyp35AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Erfassungsperiodendauer_-Wert aus Telegramm ------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        // COM: data.getUnscaledValue("WERT").set(_Messwert_);
        debug.warning("Der Typ 35 in der FG 3 wird momentan nicht untersttzt." + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 48 Id: 4 Name: UFD-Lufttemperatur LT (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 -30...+60C               0,1C                         60,0 = 600d = 0258 h
     *                                                                                0,0 = 0d = 0000 h
     *                                                                                -0,1 = -1d = FFFF h
     *                                                                                -30,0 = -300d = FED4 h
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * - 30 ... + 60 C       Temperatur, der in Nhe der Fahrbahnmessstelle befindlichen Umgebungsluft.
     *                        Messung muss weitgehend unbeeinflusst von direkter Sonneneinstrahlung und
     *                        Windeinwirkung sein.
     *                        Die Lufttemperatur wird in Grad Celsius angegeben.
     *                        Es wird der IST-Wert bertragen.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ048Id004AtgTlsUfdErgebnisMeldungLuftTemperaturLTAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_-Wert aus Telegramm -------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();
            data.getUnscaledValue("Lufttemperatur").set(_Messwert_);

	    // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
	    DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 49 Id: 4 Name: UFD-xFahrbahnoberflchentemperatur FB (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 -30...+80C               0,1C                         80,0 = 800d = 0320 h
     *                                                                                0,0 = 0d = 0000 h
     *                                                                                -0,1 = -1d = FFFF h
     *                                                                                -30,0 = -300d = FED4 h
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * - 30 ... + 80 C       Temperatur der Oberflche der Fahrbahn in Grad Celsius.
     *                        Werden andere Zustnde an der Oberflche (z.B. Benetzung) erfasst, so muss die
     *                        Oberflchentemperatur in rtlichen Bezug zu diesen Gren stehen.
     *                        Die Fahrbahnoberflchentemperatur wird in Grad Celsius angegeben.
     *                        Es wird der IST-Wert bertragen.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ049Id004AtgTlsUfdErgebnisMeldungFahrbahnOberFlchenTemperaturFBTAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_-Wert aus Telegramm -------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();
            data.getUnscaledValue("Fahrbahnoberflchentemperatur").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

	    // Ergebnisdatensatz erzeugen...
	    resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);


	    return resultData;
    }

    /**
     * FG 3 : Typ: 50 Id: 4 Name: UFD-Fahrbahnfeuchte (Antwort) TLS-Datenbeschreibung:<br> DE-Typ 50 nicht mehr fr
     * Neuanwendung verwenden. Der DE-Typ wird durch Wasserfilmdicke(DE-Typ 72) ersetzt.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ050Id004AtgTlsUfdErgebnisMeldungFahrbahnFeuchteFBFAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Fahrbahnfeuchte_ = deBlock.getDeData().readUnsignedByteAsInt();

        if ((_Fahrbahnfeuchte_ < 0) || (_Fahrbahnfeuchte_ > 100)) {
            debug.warning("Wert fr Fahrbahnfeuchte ausserhalb zulssiger Grenzwerte (0-100%)." + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());
        } else {
            data.getUnscaledValue("Fahrbahnfeuchte").set(_Fahrbahnfeuchte_);

            // Ergebnisdatensatz erzeugen...
            resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
        }

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        return resultData;
    }

    /**
     * FG 3 : Typ: 51 Id: 4 Name: UFD-Zustand der Fahrbahnoberflche FBZ (Antwort) TLS-Datenbeschreibung:<br> DE-Typ 51
     * nicht mehr fr Neuanwendung verwenden. Der DE-Typ wird durch eine neue Definition (DE-Typ 70) ersetzt.<br>
     * <pre>
     * Bit 0:   0 = Fahrbahn trocken
     *          1 = Fahrbahn na
     * Bit 1:   0 = Fahrbahn nicht schneebedeckt
     *          1 = Fahrbahn schneebedeckt
     * Bit 2,3: 0 = Fahrbahn trocken (Bit 0 = 0)
     *          1 = Feuchte 1
     *          2 = Feuchte 2
     *          3 = Feuchte 3
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ051Id004AtgTlsUfdErgebnisMeldungFahrbahnOberFlcheAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Fahrbahnoberflche").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 52 Id: 4 Name: UFD-Restsalz RS (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 8 Bit                  0%...100%                 1% Lsung                     0% = 0 d = 00 h
     *                                                                                100% = 100 d = 64 h
     *                                                                                255 = 255 d = FFh
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * 0 ... 100 %            Sttigungsgrad des gelsten Salzes auf der Fahrbahn (bzw. auf der
     *                        Fahrbahnsensorik). SR wird in Prozent (%) angegeben.
     *                        0 % bedeutet kein gelstes Salz (keine Gefrierpunkt-Erniedrigung)
     *                        100 % bedeutet vollstndig gelstes Salz, die wssrige Lsung kann kein Salz mehr
     *                        auflsen (Gefrierpunkt-Erniedrigung ist maximal).
     *                        Es wird der IST-Wert bertragen.
     * 255                    Sensorik kann auf Grund der herrschenden Bedingungen das Salzrest nicht
     *                        bestimmen.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ052Id004AtgTlsUfdErgebnisMeldungRestSalzRSAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Restsalz").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 53 Id: 4 Name: UFD-Niederschlagsintensitt NI (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 0... 200 mm/h             0,1 mm/h                      0,0 0d =0000h
     *                                                                                200 2000d =07D0h
     *
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * 0 ... 200 mm/h         Fallender Niederschlag aller Art in mm/h. Die Intensitt wird in Bezug auf
     *                        Niederschlag in flssiger Form durch Volumen (1 l/m2 = 1 mm) pro Zeiteinheit
     *                        angegeben. Ist die Sensorik in der Lage, Niederschlag in gefrorener Form zu
     *                        detektieren, wird das Wasserquivalent (Menge nach Schmelzen) verwendet.
     *                        Es wird der gleitende Mittelwert bertragen.
     * </pre>
     * Die VDI-Richtlinie VDI 3786 ist fr Niederschlagsmessungen im Bereich Streckenstationen an Straen nicht
     * anwendbar, da die in dieser Richtlinie beschriebene Messtechnik in der Regel im Straenbereich nicht anwendbar
     * ist.<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ053Id004AtgTlsUfdErgebnisMeldungNiederschlagsIntensittNIAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        data.getUnscaledValue("Niederschlagsintensitt").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 54 Id: 4 Name: UFD-Luftdruck LD (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 800...1200 hPa            1 hPa                         800 = 800 d = 0320 h
     *                                                                                1200 = 1200 d = 04B0 h
     *
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * 800 ... 1200 hPa       Atmosphrischer Luftdruck, bezogen auf Meereshhe und Normbedingung (QNH),
     *                        am Ort des Messquerschnittes in hPa.
     *                        Es wird der IST-Wert bertragen.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ054Id004AtgTlsUfdErgebnisMeldungLuftDruckLDAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        data.getUnscaledValue("Luftdruck").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 55 Id: 4 Name: UFD-Relative Luftfeuchte RLF (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 8 Bit                  10%...100%                1% rF                         10% = 10 d = 0A h
     *                                                                                100% = 100 d = 64 h
     *                                                                                1200 = 1200 d = 04B0 h
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * 10 ... 100 %           Relative Luftfeuchte der in Nhe der Fahrbahnmessstelle befindlichen
     *                        Umgebungsluft. Messung muss weitgehend unbeeinflusst von direkter
     *                        Sonneneinstrahlung und Windeinwirkung sein.
     *                        Die Rel. Feuchte sollte immer an der gleichen Stelle gemessen werden wie die
     *                        Lufttemperatur (DE-Typ 48).
     *                        Es wird der IST-Wert bertragen.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ055Id004AtgTlsUfdErgebnisMeldungRelativeLuftFeuchteRLFAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("RelativeLuftfeuchte").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 56 Id: 4 Name: UFD-Windrichtung WR (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 0...359                  1                            0 (N) = 0 d = 0000 h
     *                                                                                90(O) = 90 d = 005A h
     *                                                                                180(S) = 180 d = 00B4 h
     *                                                                                270(W) = 270 d = 010E h
     *                                                                                359 = 359 d = 0167 h
     *                                                                                65535 ...= 65535 d = FFFFh
     *
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * 0 ... 359             Richtung aus der der Wind weht in Grad der Kompassrose angegeben.
     *                        Die Windrichtung wird als vektorieller Mittelwert (Windrichtung und -
     *                        geschwindigkeit) ber ein gleitendes Zeitintervall von 10 Minuten ermittelt.
     *                        Gleichzeitig sollte auch die Windgeschwindigkeit an der gleichen Stelle gemessen
     *                        werden.
     * -1 od. 65535           Keine Windrichtung ermittelbar, da Wind zu schwach.
     * (FFFFh)                (In der Regel bei Windgeschwindigkeiten < 0,5 m/s)
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ056Id004AtgTlsUfdErgebnisMeldungWindRichtungWRAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        data.getUnscaledValue("Windrichtung").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 57 Id: 4 Name: UFD-Windgeschwindigkeit (Mittel) WGM(Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 0.0 ... 60,0 m/s          0,1 m/s                       0,0 = 0d = 0000h
     *                                                                                60,0 = 600d = 0258h
     *
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * 0 ... 60,0 m/s         Geschwindigkeit des Windes in horizontaler Ebene (Komponente) in Meter pro
     *                        Sekunde.
     *                        Windgeschwindigkeit (Mittel) wird als vektorieller Mittelwert ber ein gleitendes
     *                        Zeitintervall von 10 Minuten ermittelt.
     *                        Bei gleichzeitiger Messung der Windrichtung sind die Sensoren an der gleichen
     *                        Stelle zu platzieren.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ057Id004AtgTlsUfdErgebnisMeldungWindGeschwindigkeitMittelWertWGMAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        data.getUnscaledValue("WindgeschwindigkeitMittel").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 58 Id: 4 Name: UFD-Schneehhe(Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 8 Bit                  0 ... 200 cm              1 cm                          0 = 00d = 00h
     *                                                                                200 = 200d = C8h
     *
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * 0 ... 200 cm           Gibt die Hhe der Schneebedeckung an einer definierten Messstelle in Bezug zu
     *                        einer ebenen, senkrechten Flche an.
     *                        0 cm wird gemeldet, wenn die ebene senkrechte Flche vllig unbedeckt ist. Die
     *                        Schneeoberflche sollte durch die Messung nicht beeinflusst werden.
     *                        Die Schneehhe hat nichts mit der Bedeckung der Straenoberflche von
     *                        Fahrspuren zu tun.
     *                        Es wird der IST-Wert bertragen.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ058Id004AtgTlsUfdErgebnisMeldungSchneeHheSHAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Schneehhe").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 59 Id: 4 Name: UFD-Fahrbahngltte (Antwort) TLS-Datenbeschreibung: <br> Dieser DE-Typ wird in FG3
     * nicht definiert. Ferner bleibt es den Herstellern von Glttemeldeanlagen berlassen fr Glttezustnde und/oder
     * Glttewarnungen herstellerspezifische DE-Typ (zwischen 128 und 254) zu definieren.<br> <br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ059Id004AtgTlsUfdErgebnisMeldungFahrbahnGltteFBGAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Wert1_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _Wert2_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Wert1").set(_Wert1_);
        data.getUnscaledValue("Wert2").set(_Wert2_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 60 Id: 4 Name: UFD-Sichtweite SW (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 10 ... 1000 m             1 m                           10 = 10d = 000Ah
     *                                                                                1000 = 1000d = 03E8h
     *
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * 10 ... 1000 m          Sichtweite bezogen auf eine Kontrastschwelle von 5% an der Messstelle in Meter.
     *                        Ist diejenige Entfernung, in der zwei aneinandergrenzende Flchen, deren
     *                        Kontrastunterschied 5% betrgt, gerade noch unterschieden werden knnen.
     *                        Es wird der IST-Wert bertragen.
     * </pre>
     * Die absolute Genauigkeit auf die gesamte Sichtweite kann bis 20% betragen.<br> <br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ060Id004AtgTlsUfdErgebnisMeldungSichtWeiteSWAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        data.getUnscaledValue("Sichtweite").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 61 Id: 4 Name: UFD-Helligkeit HK (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 0.. 60000 Lx              1 Lx                          0 = 0 d = 0000 h
     *                                                                                60000 = 60000 d = EA60 h
     *
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * 0 ... 60000 Lx         Helligkeit ist definiert als Beleuchtungsstrke (Anteil des auf eine ebene Messflche
     *                        senkrecht auftreffenden Komponente des Lichtstroms im sichtbaren Spektralbereich)
     *                        in Lux angegeben.
     *                        Die Ausrichtung des Sensors sollte beachtet werden.
     *                        Es wird der IST-Wert bertragen.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ061Id004AtgTlsUfdErgebnisMeldungHelligkeitHKAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        data.getUnscaledValue("Helligkeit").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 62 Id: 4 Name: UFD-Niederschlagsmenge (alt) (Antwort) TLS-Datenbeschreibung:<br> DE-Typ 62 nicht mehr
     * fr Neuanwendungen verwenden. Statt dessen Niederschlagsintensitt (DE-Typ 53) anwenden.<br> <br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ062Id004AtgTlsUfdErgebnisMeldungNiederschlagsMengeNMAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Wert1_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _Wert2_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Wert1").set(_Wert1_);
        data.getUnscaledValue("Wert2").set(_Wert2_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 63 Id: 4 Name: UFD-Niederschlagsart (alt)(Antwort) TLS-Datenbeschreibung:<br> DE-Typ 63 nicht mehr
     * fr Neuanwendungen verwenden. Statt dessen DE-Typ 71 Niederschlagsart anwenden.<br> <br>
     * <pre>
     * Bit 0:   0 = Fahrbahn trocken
     *          1 = Fahrbahn na
     * Bit 1:   0 = Fahrbahn nicht schneebedeckt
     *          1 = Fahrbahn schneebedeckt
     * Bit 2,3: 0 = Fahrbahn trocken (Bit 0 = 0)
     *          1 = Regen 1
     *          2 = Regen 2
     *          3 = Regen 3
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ063Id004AtgTlsUfdErgebnisMeldungNiederschlagAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Niederschlag").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 64 Id: 4 Name: UFD-Windgeschwindigkeit (Spitze) WGS (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 0.0 ... 60,0 m/s          0,1 m/s                       0,0 = 0d = 0000h
     *                                                                                60,0 = 600d = 0258h
     *
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * 0 ... 60,0 m/s         Geschwindigkeit des Windes in horizontaler Ebene (Komponente) in Meter pro
     *                        Sekunde.
     *                        Windgeschwindigkeit (Spitze) ist die maximale Windgeschwindigkeit der letzten 10
     *                        Minuten.
     *                        Bei gleichzeitiger Messung der Windrichtung sind die Sensoren an der gleichen
     *                        Stelle zu platzieren.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ064Id004AtgTlsUfdErgebnisMeldungWindGeschwindigkeitSpitzenWertWGSAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        data.getUnscaledValue("WindgeschwindigkeitSpitze").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 65 Id: 4 Name: UFD-Gefriertemperatur GT(Antwort) TLS-Datenbeschreibung: </pre> <pre> Format
     * Bereich                   Auflsung                     Codierung 16 Bit                 -30...0 C 0,1C 0,0 =
     * 0d = 0000 h -0,1 = -1d = FFFF h -30,0 = -300d = FED4 h
     * <p/>
     * Inhalt bzw.            Definition Ausprgung
     * <p/>
     * - 30 ... - 0.1 C      Die Gefriertemperatur ist diejenige Temperatur in Grad Celsius, bei der die an der
     * Oberflche der Sonde vorhandene wssrige Taumittel-Lsung gefrieren (bergang in den festen Aggregatszustand)
     * knnte. Es wird der IST-Wert bertragen. 0 C                   Keine Gefriertemperatur-Erniedrigung bzw. keine
     * Taumittelkonzentration messbar bzw. die Bedingungen lassen eine Bestimmung nicht zu. Wenn gleichzeitig auch
     * Salzrest (DE-Typ 52) bertragen wird ist auch dieser Wert zu betrachten (255 = nicht messbar). </pre> Es sollte
     * unbedingt beachtet werden, dass mit keinem Messverfahren (passiv oder aktiv) die Taumittelmenge auf der
     * Oberflche und damit die bei Lsung zu erwartende Gefrierpunktserniedrigung (die zudem von der Benetzung abhngig
     * ist) bestimmt werden kann, wenn nicht eine ausreichende Lsung vorliegt. Daher ist im trockenen oder
     * abtrocknenden Zustand (bei Benetzungen unter 50 ml/m2) in der Regel keine Messung mglich. Es sollte daher fr
     * den Salzrest 255 und fr die Gefrierpunktserniedrigung 0 C gemeldet werden.<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ065Id004AtgTlsUfdErgebnisMeldungGefrierTemperaturGTAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();

        data.getUnscaledValue("Gefriertemperatur").set(_Messwert_);

	    // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
	    DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 66 Id: 4 Name: UFD-Taupunkttemperatur TPT (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 - 30 ... + 60 C          0,1 C                        60,0 = 600d = 0258h
     *                                                                                 0,0 = 0d = 0000h
     *                                                                                -0,1 = -1d = FFFFh
     *                                                                                 -30,0 = -300d = FED4h
     *
     * Inhalt bzw.            Definition
     * Ausprgung
     *
     * - 30 ... +60 C        Taupunkttemperatur ist diejenige Temperatur, bei der die in der Umgebungsluft der
     *                        Messstelle (am Ort der Temperatur und Feuchte-Sensorik) befindliche Feuchte zu
     *                        kondensieren beginnt.
     *                        Die Taupunkttemperatur ist aus der Temperatur und der relativen Feuchte der Luft
     *                        zu bestimmen.
     *                        Zur Bestimmung ist die in den neuesten betriebstechnischen Anforderungen fr
     *                        Glttemeldeanlagen enthaltene Berechnungsvorschrift zu verwenden. Dieser
     *                        Berechnungsvorschrift liegen auch die im Wetterdienst verwendeten
     *                        Taupunkttabellen zugrunde.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ066Id004AtgTlsUfdErgebnisMeldungTaupunktTemperaturTPTAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();

	    data.getUnscaledValue("Taupunkttemperatur").set(_Messwert_);

	    // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
	    DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 67 Id: 4 Name: UFD-Bodentemperatur in Tiefe 1 TT1 (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 - 30 ... + 80 C          0,1 C                        80,0 = 800d = 0320h
     *                                                                                 0,0 = 0d = 0000h
     *                                                                                -0,1 = -1d = FFFFh
     *                                                                                -30,0 = -300d = FED4h
     *
     * Inhalt bzw.            Definition
     * Ausprgung
     * - 30 ... + 80 C       Temperatur gemessen in der Tiefe 1 unter der Fahrbahn. Ist eine Fahrbahnsonde in die
     *                        Oberflche eingelassen, so befindet sich die Messstelle direkt unter der Fahrbahnsonde
     *                        in der betreffenden Tiefe
     *                        Es wird der IST-Wert bertragen.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ067Id004AtgTlsUfdErgebnisMeldungTemperaturInTiefe1TT1AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();

        data.getUnscaledValue("BodentemperaturInTiefe1").set(_Messwert_);

	    // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
	    DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 68 Id: 4 Name: UFD-UFD-Bodentemperatur in Tiefe 2 TT2 (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 - 30 ... + 80 C          0,1 C                        80,0 = 800d = 0320h
     *                                                                                 0,0 = 0d = 0000h
     *                                                                                -0,1 = -1d = FFFFh
     *                                                                                -30,0 = -300d = FED4h
     *
     * Inhalt bzw.            Definition
     * Ausprgung
     * - 30 ... + 80 C       Temperatur gemessen in der Tiefe 1 unter der Fahrbahn. Ist eine Fahrbahnsonde in die
     *                        Oberflche eingelassen, so befindet sich die Messstelle direkt unter der Fahrbahnsonde
     *                        in der betreffenden Tiefe
     *                        Es wird der IST-Wert bertragen.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ068Id004AtgTlsUfdErgebnisMeldungTemperaturInTiefe2TT2AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();

	    data.getUnscaledValue("BodentemperaturInTiefe2").set(_Messwert_);

	    // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
	    DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 69 Id: 4 Name: UFD-UFD-Bodentemperatur in Tiefe 3 TT3 (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 Bit                 - 30 ... + 80 C          0,1 C                        80,0 = 800d = 0320h
     *                                                                                 0,0 = 0d = 0000h
     *                                                                                -0,1 = -1d = FFFFh
     *                                                                                -30,0 = -300d = FED4h
     *
     * Inhalt bzw.            Definition
     * Ausprgung
     * - 30 ... + 80 C       Temperatur gemessen in der Tiefe 1 unter der Fahrbahn. Ist eine Fahrbahnsonde in die
     *                        Oberflche eingelassen, so befindet sich die Messstelle direkt unter der Fahrbahnsonde
     *                        in der betreffenden Tiefe
     *                        Es wird der IST-Wert bertragen.
     * </pre>
     * Die Tiefe 1 liegt der Oberflche immer am nchsten. Es gilt Tiefe 1 < Tiefe 2 < Tiefe 3. Tiefe 3 sollte immer 30
     * cm betragen. Wenn nur eine Bodentiefentemperatur gemessen wird, sollte immer als Tiefe 30 cm (DE-Typ 69)
     * verwendet werden.<br> <br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ069Id004AtgTlsUfdErgebnisMeldungTemperaturInTiefe3TT3AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();

        data.getUnscaledValue("BodentemperaturInTiefe3").set(_Messwert_);

	    // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
	    DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 70 Id: 4 Name: UFD-Zustand der Fahrbahnoberflche FBZ (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Inhalt bzw.            Definition
     * Ausprgung
     * 0                      Fahrbahn ist vollkommen Trocken (< ca. 30 ml/m2 = 0,03 mm), schnee- und eisfrei
     * 1                      Fahrbahn ist feucht bzw. nass, oder schnee- oder eisbedeckt. Die Benetzung bzw.
     *                        Bedeckung bersteigt ca. 30 ml/m2 = 0,03 mm.
     *                        Nhere Differenzierung der Bedeckungsart nicht mglich.
     * 2 ... 31               frei fr Erweiterungen
     * 32                     Fahrbahn ist benetzt mit flssigem Wasser bzw. wssriger Lsung.
     *                        Die Menge bersteigt ca. 30 ml/m2 = 0,03 mm.
     *                        Nhere Differenzierung nicht mglich.
     * 33 ... 63              frei fr Erweiterungen
     * 64                     Fahrbahn ist bedeckt mit gefrorenem Wasser bzw. wssriger Lsung in festem
     *                        Zustand. Weitere Differenzierung nicht mglich.
     * 65                     Fahrbahn ist bedeckt mit Schnee oder Schneematsch. Gemisch von flssigem und
     *                        gefrorenem Wasser bzw. wssriger Lsung.
     * 66                     Fahrbahn ist bedeckt mit Eis (festes, gefrorenes Wasser bzw. gefrorene wssrige
     *                        Lsung)
     * 67                     Fahrbahn ist bedeckt mit Rauhreif. Aus der Luft sublimierte Eiskristalle ohne
     *                        deckende Eisflche. Die Taupunkttemperatur liegt nahe der Fahrbahnoberflchen-
     *                        Temperatur und liegt unter der Gefriertemperatur.
     * 68 ... 127             frei fr Erweiterungen
     * 128 ... 254            frei fr hersteller- oder anwendungsspezifische Codes
     * 255                    Sensorik kann auf Grund der herrschenden Bedingungen Zustand nicht bestimmen.
     * </pre>
     * Es ist zu beachten, dass in Relation zur Strecke die Messung immer nur punktfrmig sein kann und daher die
     * Ergebnisse entsprechend interpretiert werden mssen.<br> Die Benetzung bzw. Bedeckung wird immer bezogen auf eine
     * glatte ebene Flche. Die Einschtzung der Gefhrlichkeit einer Benetzung oder Bedeckung in Bezug zur
     * Beschaffenheit des Fahrbahnbelags (Rauhigkeit etc.) und den besonderen Bedingungen der betreffenden
     * Straenstrecke muss in der Zentrale vorgenommen werden.<br> Es ist nicht erforderlich, dass die Sensortechnik
     * alle Zustnde direkt als solches detektieren kann. Es gengt vielmehr, wenn die zur Bildung der Zustnde
     * beitragenden Einflsse gemessen werden knnen, so dass der betreffende Zustand als wahrscheinlich angenommen
     * werden kann.<br> Von einer Fahrbahnmessstelle mssen nicht alle Ausprgungen untersttzt werden.<br> Die Menge
     * der Benetzung bzw. Bedeckung, soweit bestimmbar, wird durch die Wasserfilmdicke (mm bzw. l/m2 ) angegeben.<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ070Id004AtgTlsUfdErgebnisMeldungZustandDerFahrbahnOberFlcheFBZAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("ZustandFahrbahnoberflche").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 71 Id: 4 Name: UFD-Niederschlagsart (NS)(Antwort) TLS-Datenbesc hreibung:
     * <pre>
     * Inhalt bzw.            Definition
     * Ausprgung
     * alle                   In der Atmosphre fallender Niederschlag.
     *                        Wird gleichzeitig Niederschlagsintensitt ermittelt, ist diese an der gleichen Stelle
     *                        vorzunehmen.
     *                        Es werden die Klassifikation und die Codes nach WMO Tabelle 4680 verwendet.
     * 0                      kein Niederschlag
     * 1 ... 39               nicht benutzen
     * 40                     Niederschlag aller Art
     *                        Nicht nher Klassifizier- und quantifizierbar oder Sensorik nicht dafr ausgelegt
     * 41                     Leichter oder mittlerer Niederschlag aller Art ( < 50 Partikel/Minute)
     * 42                     Starker Niederschlag aller Art ( > 50 Partikel/Minute)
     * 43 ... 49              frei fr Erweiterungen
     * 50                     Sprhregen
     *                        (keine weitere Klassifikation von Sprhregen mglich)
     * 51 ... 59              weitere Klassifikation von Sprhregen n. WMO
     * 60                     Regen bzw. flssiger Niederschlag
     *                        (Keine weitere Klassifikation von Regen mglich)
     * 61 ... 69              Weitere Klassifikation von Regen n. WMO
     * 70                     Schnee bzw. gefrorener Niederschlag
     *                        (Keine weitere Klassifikation von gefrorenem Niederschlag mglich)
     * 71 ... 73              Weiter Klassifikation von Schnee n. WMO
     * 74 ... 76              Weitere Klassifikation von Graupel n. WMO
     * 77 ... 79              Weitere Klassifikation von Hagel n. WMO
     * 80 ... 127             frei fr Erweiterungen
     * 128 ... 254            frei fr hersteller- und anwendungsspez. Erweiterungen
     * 255                    Sensorik kann auf Grund der herrschenden Bedingungen Zustand nicht bestimmen
     * </pre>
     * Es wird darauf hingewiesen, dass von der Streckenstation bzw. der Sensorik nicht alle Ausprgungen untersttzt
     * werden mssen. Welcher Differenzierungsgrad notwendig und sinnvoll ist, hngt von der Anwendung ab. Bei einfachen
     * Anwendungen knnen die Ausprgungen 0 und 40, bei normalen Anforderungen 0, 60 und 70 ( entspricht den bisher
     * verwendeten Ausprgungen 00, 01, 02 des Typs 63 ) ausreichen.<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ071Id004AtgTlsUfdErgebnisMeldungNiederschlagsArtNSAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Niederschlagart").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 3 : Typ: 72 Id: 4 Name: UFD-Wasserfilmdicke WFD (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 16 bit                 0,00...10,00 mm           0,01 mm                       0 = 0d = 0000h
     *                                                                                10,00 = 1000d = 03E8h
     *                                                                                65535 = 65535d = FFFFh
     * </pre>
     * Bei WFD oberhalb 0,1 mm wird nur eine Messgenauigkeit im 1/10 mm Bereich erreicht. Bei WFD oberhalb 2 mm liegt
     * die Messgenauigkeit in der Regel im mm Bereich<br>
     * <pre>
     * Inhalt bzw.            Definition
     * Ausprgung
     * 0 ... 10,00 mm         Die rechnerische Wasserfilmdicke gibt die Fahrbahnbenetzung in mm (= Liter/m2 ) mit
     *                        Wasser oder wssriger Lsung (Salzgemisch) an, welche sich am Messort bzw. auf der
     *                        in der Fahrbahn eingelassenen Sonde befindet. Die Wasserfilmdicke ist immer bezogen
     *                        auf eine glatte ebene Flche.
     * -1 od. 65535           Wasserfilmdicke auf Grund der herrschenden Bedingungen nicht bestimmbar oder feste
     * (FFFFh)                bzw. gefrorene Bedeckung.
     * </pre>
     * Es ist zu beachten, dass in Relation zur Strecke die Messung immer nur punktfrmig sein kann und daher die
     * Ergebnisse entsprechend interpretiert werden mssen.<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ072Id004AtgTlsUfdErgebnisMeldungWasserFilmDickeWFDAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        data.getUnscaledValue("Wasserfilmdicke").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    public ResultData Fg003Typ073Id004AtgTlsUfdErgebnisMeldungTaustoffkonzentrationTSKAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Taustoffkonzentration").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    public ResultData Fg003Typ074Id004AtgTlsUfdErgebnisMeldungTaustoffmengeTSQAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Taustoffmenge").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    public ResultData Fg003Typ076Id04AtgTlsUfdErgebnisMeldungEisFilmDickeEFDAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Eisfilmdicke").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    public ResultData Fg003Typ079Id004AtgTlsUfdErgebnisMeldungZustandDerFahrbahnOberFlcheWinterdienstFZWAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("ZustandFahrbahnoberflcheWinterdienst").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }


    /**
     * Entspricht {@link de.bsvrz.kex.tls.osi7.conversion.Fg003Default#Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterIst(DeBlock,
     * Data, DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg255Typ029Id034AtgTlsGloKanalSteuerungAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * Entspricht {@link de.bsvrz.kex.tls.osi7.conversion.Fg003Default#Fg255Typ036Id003AtgTlsGloGeographischeKenndatenAspTlsZustand(DeBlock,
     * Data, DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg255Typ036Id035AtgTlsGloGeographischeKenndatenAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg255Typ036Id003AtgTlsGloGeographischeKenndatenAspTlsZustand(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * Standardumsetzungsmethode, um {@link de.bsvrz.kex.tls.osi7.DeBlock}-Daten in {@link ResultData} eines logischen
     * Ufd-Sensors ("atg.ufdsXXX", "asp.externeErfassung", "typ.ufdsXXX") zu konvertieren.
     *
     * @param messWert         Der Messwert des TLS-Sensors (unskaliert!!!)
     * @param data             Das #{@link de.bsvrz.dav.daf.main.Data}-Objekt fr die Zieldaten.
     * @param dataDescription  Die #{@link DataDescription} fr die Zieldaten.
     * @param att              Der Name der Attributliste (z. B. "LuftTemperatur", der den eigentlichen Messwert enthlt
     *                         (DaV-seitig).
     * @param nichtErmittelbar Der Messwert, der von einem TLS-Sensor fr nicht ermittelbare Daten gesetzt wird, i. d.
     *                         R. <code>255</code>, <code>32767</code> oder <code>65535</code>.
     * @param intervallDauer   Die Erfassungsintervalldauer in Sekunden des TLS-Sensors. Bei einem negativen Wert wird
     *                         automatisch die Standardvorgabe eingestellt (i. d. R. 60 Sekunden).
     *
     * @return Der Ergebnisdatensatz mit den konvertierten MeteoDaten.
     */
    protected ResultData tlsDatenNachResultUfdLogisch(int messWert, Data data, DataDescription dataDescription, String att, int nichtErmittelbar, int intervallDauer) {
        // Die Ausfhrung von Umsetzungsmethoden kann zur Laufzeit ber den
        // Parameter atg.tlsPassivierungbernahmeLogischesModell am typ.de
        // unterbunden werden, in dem der Zustand des Parameters ausgewertet
        // und bei Passivierung als return-Wert null zurckgegeben wird
        if (isPassivierungUmsetzungsMethoden()) {
            return null;
        }


        try {
            final int intervallDauerStandard = 60;  // Standardintervalllnge 1 Minute (60 Sekunden).

            if (intervallDauer < 0) {
                data.getTimeValue("T").setSeconds(intervallDauerStandard);
            } else {
                data.getTimeValue("T").setSeconds(intervallDauer);
            }

            Data attListe = data.getItem(att);      // Attributliste fr Umfelddatensensoren logisch.

            // Messwert ermitteln, dazu Messwert skalieren und eventuelle Fehlerzustnde bercksichtigen.
            if (messWert == nichtErmittelbar) {
                attListe.getTextValue("Wert").setText("nicht ermittelbar");
            }

            // else if (!isGueltigerMessWert(dataDescription, att, true, messWert, skalierung)) {
            // attListe.getTextValue("Wert").setText("nicht ermittelbar/fehlerhaft");
            // }
            else {
                attListe.getUnscaledValue("Wert").set(messWert);
            }

            attListe.getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").set(0);
            attListe.getItem("Status").getItem("PlFormal").getUnscaledValue("WertMax").set(0);
            attListe.getItem("Status").getItem("PlFormal").getUnscaledValue("WertMin").set(0);
            attListe.getItem("Status").getItem("PlLogisch").getUnscaledValue("WertMaxLogisch").set(0);
            attListe.getItem("Status").getItem("PlLogisch").getUnscaledValue("WertMinLogisch").set(0);
            attListe.getItem("Status").getItem("MessWertErsetzung").getUnscaledValue("Implausibel").set(0);
            attListe.getItem("Status").getItem("MessWertErsetzung").getUnscaledValue("Interpoliert").set(0);
            attListe.getItem("Gte").getScaledValue("Index").set(1.0);
            attListe.getItem("Gte").getUnscaledValue("Verfahren").set(0);

        }
        catch (Exception e) {
            debug.warning("TLSDatenNachResultUfdLogisch, Konvertierungsfehler: " + e);
            return null;
        }

        // Ergebnisdatensatz erzeugen.
        return new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
    }

    /**
     * Standardumsetzungsmethode, um {@link de.bsvrz.kex.tls.osi7.DeBlock}-Daten mit 32-Bit Gleitkommawerten in {@link
     * ResultData} eines logischen Ufd-Sensors ("atg.ufdsXXX", "asp.externeErfassung", "typ.ufdsXXX") zu konvertieren.
     *
     * @param messWert        Der Messwert des TLS-Sensors (als Gleitkommazahl 32 Bit, einfache Genauigkeit)
     * @param data            Das #{@link de.bsvrz.dav.daf.main.Data}-Objekt fr die Zieldaten.
     * @param dataDescription Die #{@link DataDescription} fr die Zieldaten.
     * @param att             Der Name der Attributliste (z. B. "LuftTemperatur", der den eigentlichen Messwert enthlt
     *                        (DaV-seitig).
     * @param intervallDauer  Die Erfassungsintervalldauer in Sekunden des TLS-Sensors. Bei einem negativen Wert wird
     *                        automatisch die Standardvorgabe eingestellt (i. d. R. 60 Sekunden).
     *
     * @return Der Ergebnisdatensatz mit den konvertierten MeteoDaten.
     */
    protected ResultData tlsDatenNachResultUfdLogischGleitkomma(float messWert, Data data, DataDescription dataDescription, String att, int intervallDauer) {
        // Die Ausfhrung von Umsetzungsmethoden kann zur Laufzeit ber den
        // Parameter atg.tlsPassivierungbernahmeLogischesModell am typ.de
        // unterbunden werden, in dem der Zustand des Parameters ausgewertet
        // und bei Passivierung als return-Wert null zurckgegeben wird
        if (isPassivierungUmsetzungsMethoden()) {
            return null;
        }

        try {
            final int intervallDauerStandard = 60;  // Standardintervalllnge 1 Minute (60 Sekunden).

            if (intervallDauer < 0) {
                data.getTimeValue("T").setSeconds(intervallDauerStandard);
            } else {
                data.getTimeValue("T").setSeconds(intervallDauer);
            }

            Data attListe = data.getItem(att);      // Attributliste fr Umfelddatensensoren logisch.

            attListe.getScaledValue("Wert").set(messWert);
            attListe.getItem("Status").getItem("Erfassung").getUnscaledValue("NichtErfasst").set(0);
            attListe.getItem("Status").getItem("PlFormal").getUnscaledValue("WertMax").set(0);
            attListe.getItem("Status").getItem("PlFormal").getUnscaledValue("WertMin").set(0);
            attListe.getItem("Status").getItem("PlLogisch").getUnscaledValue("WertMaxLogisch").set(0);
            attListe.getItem("Status").getItem("PlLogisch").getUnscaledValue("WertMinLogisch").set(0);
            attListe.getItem("Status").getItem("MessWertErsetzung").getUnscaledValue("Implausibel").set(0);
            attListe.getItem("Status").getItem("MessWertErsetzung").getUnscaledValue("Interpoliert").set(0);
            attListe.getItem("Gte").getScaledValue("Index").set(1.0);
            attListe.getItem("Gte").getUnscaledValue("Verfahren").set(0);
        }
        catch (Exception e) {
            debug.warning("TLSDatenNachResultUfdLogischGleitkomma, Konvertierungsfehler: " + e);
            return null;
        }

        // Ergebnisdatensatz erzeugen.
        return new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 19.01.13
